package Currency is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   2.0
  -- Date      31 August 2017
  --====================================================================
  -- Package for Euro currency management:
  -- A given amount can be divided into coins and banknotes (in the
  -- minimal possible number), and the amount of a purse's contents in
  -- coins and banknotes can be output.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    2.0  31.08.2017 English version
  --  C.G.    1.1  20.03.2014 Adapted for Ada tasking course
  --  C.G.    1.0  01.10.2013 Created for Ada course
  --====================================================================

  type Piece is
    (Cent_1, Cent_2, Cent_5, Cent_10, Cent_20, Cent_50,
     Euro_1, Euro_2, Euro_5, Euro_10, Euro_20, Euro_50, Euro_100, Euro_200, Euro_500);
  subtype Coin     is Piece range Cent_1 .. Euro_2;
  subtype Banknote is Piece range Euro_5 .. Euro_500;

  type Denomination is array (Piece range <>) of Natural;

  Value: constant array (Piece) of Positive :=
    (Cent_1   =>      1,
     Cent_2   =>      2,
     Cent_5   =>      5,
     Cent_10  =>     10,
     Cent_20  =>     20,
     Cent_50  =>     50,
     Euro_1   =>   1_00,
     Euro_2   =>   2_00,
     Euro_5   =>   5_00,
     Euro_10  =>  10_00,
     Euro_20  =>  20_00,
     Euro_50  =>  50_00,
     Euro_100 => 100_00,
     Euro_200 => 200_00,
     Euro_500 => 500_00);

  function Itemize (Amount: Natural     ) return Denomination;
  function Amount  (Pieces: Denomination) return Natural;

end Currency;
