with Ticket_Automaton.Languages;

separate (Ticket_Automaton)
procedure Create_GUI is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.5
  -- Date      27 April 2022
  --====================================================================
  -- Executed during elaboration.
  --
  -- It's extremely cumbersome to get a halfway nice layout.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for Change
  --  C.G.    1.5  27.04.2022 Correct spelling of Set_Text_Aligbnment
  --                          as afforded by download of Ada_GUI today;
  --                          Set_Read_Only for Text_Boxes Ticket and
  --                          Amount (Ada_GUI issue 7)
  --  C.G.    1.4  17.04.2022 Separate row for tickets and price for
  --                          nice alignment
  --  C.G.    1.3  17.04.2022 Add title
  --  C.G.    1.2  13.04.2022 Clean up
  --  C.G.    1.1  11.04.2022 New version of Ada_GUI; polyglotism
  --  C.G.    1.0  08.04.2022 Ironing out
  --  C.G.    0.1  05.04.2022 Layout finished (not really nice)
  --  C.G.    0.0  02.04.2022 Template only (learning Ada_GUI)
  --====================================================================

  use Ada_GUI;

  Space: Widget_ID;

  use Languages;

begin

  Languages.Initialize;

  Set_Up (ID    => 8081,  -- seems to be port number, therfore restricted range (which?)
          Title => "Ticket Automaton",  -- unchangeable                       -- Layout: (5, 4)
          Icon  => "Ada.ico",
          Grid  => (1 | 5 => (1      => (Area, Center) , 2 .. 4 => (Kind => Extension)),    -- 1:    Title
                    2     => (1 .. 4 => (Area, Center)),                                    -- 2:   Tickets
                    3 | 4 => (1      => (Area, Right ) ,                                    -- 3: Buy     Help
                              3      => (Area, Left  ) , 2 |  4 => (Kind => Extension))));  -- 4: Money   Abort
                                                                                            -- 5:   Language
  -- Headline

  Title_Line := New_Background_Text (Text => Message (Title).all, Row => 1);

  -- Ticket Choice and Price

  Short     := New_Button (Text => Message (Short_Trip)    .all, Row => 2, Column => 1);
  City      := New_Button (Text => Message (City_Region)   .all, Row => 2, Column => 2);
  Metropole := New_Button (Text => Message (Metropole_Area).all, Row => 2, Column => 3);
  Day       := New_Button (Text => Message (Day_Ticket)    .all, Row => 2, Column => 4);

  Price := New_Background_Text (Text => Image (Ticket_Price (Short_Trip))    , Row => 2, Column => 1, Break_Before => True);
  Price := New_Background_Text (Text => Image (Ticket_Price (City_Region))   , Row => 2, Column => 2, Break_Before => True);
  Price := New_Background_Text (Text => Image (Ticket_Price (Metropole_Area)), Row => 2, Column => 3, Break_Before => True);
  Price := New_Background_Text (Text => Image (Ticket_Price (Day_Ticket))    , Row => 2, Column => 4, Break_Before => True);

  -- Buy

  Ticket := New_Text_Box        (Label => Message (Selected_Ticket).all, Width => 20, Row => 3);
  Space  := New_Background_Text (Text  => "&nbsp;"                                     ,              Row => 3, Break_Before => True);
  Amount := New_Text_Box        (Label => Message (Amount_to_Pay)  .all, Width =>  8, Row => 3, Break_Before => True);
  Set_Text_Alignment (Ticket, Right);
  Set_Text_Alignment (Amount, Right);
  Set_Read_Only      (Ticket);
  Set_Read_Only      (Amount);

  -- Help

  Space := New_Background_Text (Text => 30 * "&nbsp;"                           , Row => 3, Column => 3, Break_Before => True);
  Help  := New_Button          (Text => "<font size=""+3"">&nbsp;?&nbsp;</font>", Row => 3, Column => 3);

  -- Money

  Cent_10 := New_Button          (Text => "10 &cent;"   , Row => 4);
  Space   := New_Background_Text (Text => "&nbsp;&nbsp;", Row => 4);
  Cent_20 := New_Button          (Text => "20 &cent;"   , Row => 4);
  Space   := New_Background_Text (Text => "&nbsp;&nbsp;", Row => 4);
  Cent_50 := New_Button          (Text => "50 &cent;"   , Row => 4);
  Space   := New_Background_Text (Text =>  3 * "&nbsp;" , Row => 4);
  Space   := New_Background_Text (Text => "&nbsp;"      , Row => 4, Break_Before => True);
  Euro_1  := New_Button          (Text => " 1 &euro;"   , Row => 4, Break_Before => True);
  Space   := New_Background_Text (Text => "&nbsp;&nbsp;", Row => 4);
  Euro_2  := New_Button          (Text => " 2 &euro;"   , Row => 4);
  Space   := New_Background_Text (Text => "&nbsp;&nbsp;", Row => 4);
  Euro_5  := New_Button          (Text => " 5 &euro;"   , Row => 4);
  Space   := New_Background_Text (Text => "&nbsp;&nbsp;", Row => 4);
  Euro_10 := New_Button          (Text => "10 &euro;"   , Row => 4);

  -- Abortion

  Space    := New_Background_Text (Text => 10 * "&nbsp;"             , Row => 4, Column => 3, Break_Before => True);
  Abortion := New_Button          (Text => Message (Abort_Button).all, Row => 4, Column => 3);

  -- Language

  Language := New_Button (Text => Message (Language_Button).all, Row => 5);

end Create_GUI;
