with Ada.Characters.Latin_1;
with Ada.Strings.Unbounded;
use  Ada.Strings.Unbounded;

separate (Ticket_Automaton.Run)
package body Dialogs is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.6
  -- Date      18 August 2022
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for Change
  --  C.G.    1.6  18.08.2022 Add Castillano
  --  C.G.    1.5  27.04.2022 Set_Label for Text_Boxes Ticket and
  --                          Amount (Ada_GUI issue 5)
  --  C.G.    1.4  17.04.2022 Add title
  --  C.G.    1.3  13.04.2022 Add Instructions
  --  C.G.    1.2  11.04.2022 Polyglotism
  --  C.G.    1.1  10.04.2022 Replace Dialogs.Selected_Button by
  --                          Show_Message_Box
  --  C.G.    1.0  09.04.2022
  --====================================================================

  use Languages;

  procedure Choose_Language is
    Current: constant Language_Name := Current_Language;
    Chosen : constant String        :=
      Ada_GUI.Dialogs.Selected_Button (Title  => Message (Some_Languages).all,
                                       Text   => Message (Choose_Language).all,
                                       Button => (To_Unbounded_String (Castillano'Image),
                                                  To_Unbounded_String (Deutsch   'Image),
                                                  To_Unbounded_String (English   'Image),
                                                  To_Unbounded_String (Franais  'Image),
                                                  To_Unbounded_String (Italiano  'Image)));
  begin
    if Language_Name'Value (Chosen) = Current then
      return;
    end if;
    Set (Language_Name'Value (Chosen));  -- set tentatively until acknowledged
    declare
      Ack: constant String :=
        Ada_GUI.Dialogs.Selected_Button (Title  => Message (Some_Languages).all,
                                         Text   => Message (Selected_Language).all & Chosen,
                                         Button => (To_Unbounded_String (Message (Yes).all),
                                                    To_Unbounded_String (Message (No) .all)));
    begin
      if Ack = Message (No).all then  -- rejected choice
        Set (Current);  -- revert choice
        return;
      end if;
    end;
    Title_Line.Set_Text  (Message (Title)          .all);
    Short     .Set_Text  (Message (Short_Trip)     .all);
    City      .Set_Text  (Message (City_Region)    .all);
    Metropole .Set_Text  (Message (Metropole_Area) .all);
    Day       .Set_Text  (Message (Day_Ticket)     .all);
    Ticket    .Set_Label (Message (Selected_Ticket).all);
    Amount    .Set_Label (Message (Amount_to_Pay)  .all);
    Abortion  .Set_Text  (Message (Abort_Button)   .all);
    Language  .Set_Text  (Message (Language_Button).all);
  end Choose_Language;

  function Handle_Money (Pieces: Denomination; Text: String) return String is
    Result: Unbounded_String := Null_Unbounded_String;
  begin
    if Pieces = Empty then
      return To_String (Result);
    end if;
    Result := To_Unbounded_String (Text & Image (C.Amount (Pieces)));
    for I in Pieces'Range loop
      if Pieces (I) > 0 then
        Result := @ & Ada.Characters.Latin_1.LF &
                  Piece'Image (I) & " =>" & Pieces (I)'Image;
      end if;
    end loop;
    return To_String (Result);
  end Handle_Money;

  procedure Accomplished (Selected: Ticket_Kind; Change: Denomination) is
  begin
    Show_Message_Box (Message (Accomplished ).all & Ada.Characters.Latin_1.LF &
                      Message (Remove_Ticket).all & Selected'Image &
                      Handle_Money (Change, Message (Remove_Change).all));
  end Accomplished;

  procedure Abandonment (Inserted: Denomination; Reason: String) is
  begin
    Show_Message_Box (Message (Purchase_Aborted).all & Reason & Ada.Characters.Latin_1.LF &
                      Handle_Money (Inserted, Message (Remove_Money).all));
  end;

  procedure Instructions is
  begin
    Show_Message_Box (Message (Instructions).all & Ada.Characters.Latin_1.LF &
                      Message (Happiness).all);
    Show_Message_Box (Message (Terms).all & Ada.Characters.Latin_1.LF &
                      Message (Authorisation).all);
  end Instructions;

end Dialogs;
