separate (Ticket_Automaton.Run)
package body Scrooge is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      8 May 2022
  --====================================================================
  -- Eponyms:
  -- * Scrooge McDuck, in German Dagobert Duck
  -- * Beagle Boys, in German Panzerknacker
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.1  08.05.2022 Also count failures
  --  C.G.    1.0  09.04.2022 Taken from GtkAda implementation
  --====================================================================

  Thaler: Denomination :=  -- freshly filled machine contains coins and notes
    (C.Cent_10 => 20,      -- for change; here sparce contents in order
     C.Cent_20 => 10,      -- to provoke change problems early on
     C.Cent_50 =>  5,
     C.Euro_1  => 10,
     C.Euro_2  =>  5,
     C.Euro_5  =>  5,
     C.Euro_10 =>  0);     -- impossible as change

  -- Replace piece with next smaller ones
  E5: constant Denomination := (C.Euro_2  => 2, C.Euro_1  => 1, others => 0);
  E2: constant Denomination := (C.Euro_1  => 2,                 others => 0);
  E1: constant Denomination := (C.Cent_50 => 2,                 others => 0);
  C5: constant Denomination := (C.Cent_20 => 2, C.Cent_10 => 1, others => 0);
  C2: constant Denomination := (C.Cent_10 => 2,                 others => 0);

  Trade: Tickets_Sold := (others => 0);

  --------------------------------------------------------------------------

  function "+" (L, R: Denomination) return Denomination
    with Inline
  is
  begin
    return Result: Denomination do
      for M in Result'Range loop
        Result (M) := L (M) + R (M);
      end loop;
    end return;
  end "+";

  function "*" (L: Natural; R: Denomination) return Denomination
    with Inline
  is
  begin
    return Result: Denomination do
      for M in Result'Range loop
        Result (M) := L * R (M);
      end loop;
    end return;
  end "*";

  procedure Adapt (M: in Coin_or_Note; Difference: in Natural; Change: in out Denomination)
    with Inline
  is
    -- Split missing pieces into the next smaller ones
  begin
    Change := (case M is
                 when C.Euro_5  => Change + Difference * E5,
                 when C.Euro_2  => Change + Difference * E2,
                 when C.Euro_1  => Change + Difference * E1,
                 when C.Cent_50 => Change + Difference * C5,
                 when C.Cent_20 => Change + Difference * C2,
                 when others  => Change);
  end Adapt;

  --------------------------------------------------------------------------

  procedure Store (Selected      : in  Ticket_Kind;
                   To_be_Changed : in  Natural;
                   Paid          : in  Denomination;
                   Change        : out Denomination;
                   Change_Problem: out Boolean) is
    Contents  : Denomination := Thaler + Paid;
    Difference: Natural;
  begin
    Change := Itemize (To_be_Changed) (Coin_or_Note);
    if To_be_Changed > 0 then
      for M in reverse Contents'Range loop
        if Change (M) <= Contents (M) then
          Contents (M) := Contents (M) - Change (M);  -- enough piecese M in store
        else
          if M = C.Cent_10 then
            Change_Problem := True;
            Change         := Paid;  -- not enough pieces in store
            Trade (None)   := @ + 1;
            return;
          end if;
          Difference := Change (M) - Contents (M);  -- this is missing
          Change (M) := Contents (M);               -- take as much as there is
          Adapt (M, Difference, Change);            -- and for the rest take smaller pieces
        end if;
      end loop;
    end if;
    Change_Problem   := False;
    Thaler           := Contents;
    Trade (Selected) := @ + 1;
  end Store;

  procedure Beagle_Boys (Sales: out Tickets_Sold;
                         Store: out Denomination) is
  begin
    Sales := Trade;
    Store := Thaler;
  end Beagle_Boys;

end Scrooge;
