with Ada_GUI;

with Currency;
use  Currency;

package body Ticket_Automaton is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      17 April 2022
  --====================================================================
  -- Ticket automaton with Ada_GUI
  --====================================================================
  -- History
  -- Author Version   Date    Reason for Change
  --  C.G.    1.1  17.04.2022 Add title
  --  C.G.    1.0  09.04.2022 Flesh
  --  C.G.    0.0  02.04.2022 Template only (learning Ada_GUI)
  --====================================================================

  -- Money

  function Image (Money: Integer) return String is
    Raw: constant String := Money'Image;  -- minimum length 3 (for e.g. 10 Cent)
  begin
    if Money <= 0 then
      return "";  -- only positive values will be displayed
    else      -- fore
      return (if Raw (Raw'Last - 2) = ' ' then " 0" else Raw (1 .. Raw'Last - 2)) & '.' &
              -- aft
                 Raw (Raw'Last - 1 .. Raw'Last) & " &euro;";
    end if;
  end Image;

  -- Ticket

  type Ticket_Choice  is (None, Short_Trip, City_Region, Metropole_Area, Day_Ticket);
  subtype Ticket_Kind is Ticket_Choice range Short_Trip .. Day_Ticket;

  Ticket_Price: constant array (Ticket_Kind) of Integer :=
    (Short_Trip     =>  1_50,
     City_Region    =>  3_00,
     Metropole_Area =>  6_70,
     Day_Ticket     => 11_10);

  -- Buttons and Text Boxes arranged following layout:
  -- 1:    Ticket
  -- 2: Buy     Help
  -- 3: Money   Abort
  -- 4:   Language

  Title_Line: Ada_GUI.Widget_ID;

  Short, City, Metropole, Day: Ada_GUI.Widget_ID;
  Price: Ada_GUI.Widget_ID;

  Ticket, Amount: Ada_GUI.Widget_ID;

  Help: Ada_GUI.Widget_ID;

  Cent_10, Cent_20, Cent_50, Euro_1, Euro_2, Euro_5, Euro_10: Ada_GUI.Widget_ID;
  Abortion: Ada_GUI.Widget_ID;

  Language: Ada_GUI.Widget_ID;

  -- GUI in Action

  procedure Run        is separate;
  procedure Create_GUI is separate;

begin

  Create_GUI;

end Ticket_Automaton;
