--with Ada.Text_IO;  -- fr Timer-Test
with Ada.Characters.Latin_1, Ada.Strings.Fixed;

with Fahrkartenautomat.Ausgabeschacht;
with Fahrkartenautomat.Dagobert;

separate (Fahrkartenautomat)
package body Callback is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.3
  -- Datum     19 September 2021
  --====================================================================
  --
  --====================================================================
  -- Geschichte
  -- Autor  Version   Datum   Grund der nderung
  --  C.G.    1.3  19.09.2021 Abbruch bei Zeitberschreitung
  --  C.G.    1.2  15.09.2021 Revision implementiert
  --  C.G.    1.1  12.09.2021 Character'Pos (164) als Euro-Platzhalter
  --  C.G.    1.0  11.09.2021 Es fehlt noch: Leerung des Automaten bei
  --                          Schlieung des Automaten
  --====================================================================

  LF: Character renames Ada.Characters.Latin_1.LF;
  function "*" (N: Natural; C: Character) return String renames Ada.Strings.Fixed."*";

  procedure Zeige (Titel, Nachricht: String) is separate;

  procedure Abbruch (Grund: String);

  -- Gpbae Daten

  Gewhlt: Fahrkartenoption := Keine;
  Gezahlt: Stckelung;
  Zahlbar: Integer;

  -- Kartenkauf

  procedure Karte (Self: access Gtk.Button.Gtk_Button_Record'Class) is
    Karte: constant Fahrkarte := Fahrkarte'Value (Gtk.Button.Get_Label (Self));
  begin
    if Gewhlt = Keine then
      Gewhlt := Karte;
      Zahlbar := Preis (Karte);
      Gezahlt := (others => 0);
      Gtk.GEntry.Set_Text (Wahl  ,  Image (Karte  ));  -- Wie zentrieren?
      Gtk.GEntry.Set_Text (Betrag, +Image (Zahlbar));
    end if;
  end Karte;

  -- Geldeinwurf

  -- Es erscheint recht umstndlich, aus Self das akzeptierte Geldstck zu
  -- erhalten. Daher diese Lsung.
  generic
    Stck: Akzeptiertes_Geldstck;
  procedure Einwurf (Self: access Gtk.Button.Gtk_Button_Record'Class);

  procedure Einwurf (Self: access Gtk.Button.Gtk_Button_Record'Class) is
  begin
    if Gewhlt /= Keine and then Zahlbar > 0 then
      Gezahlt (Stck) := @ + 1;
      Zahlbar         := @ - Wert (Stck);
      Gtk.GEntry.Set_Text (Betrag, +Image (Zahlbar));
      if Zahlbar <= 0 then  --Kauf zuende
       Stop_Timeout (Self);
       declare
          Wechselgeld   : Stckelung;
          Wechselproblem: Boolean;
        begin
          Dagobert.Speichere (Gewhlt,
                              -Zahlbar, Gezahlt,
                              Wechselgeld, Wechselproblem);
          if Wechselproblem then
            Abbruch ("wegen Wechselproblem");
            return;
          end if;
          Zeige (Titel     => "Fahrkartenausgabe",
                 Nachricht => "Entnehmen Sie die Fahrkarte" &
                              (if Zahlbar = 0 then ""
                                              else LF & " und das Wechselgeld" & Image (-Zahlbar)));
          Ausgabeschacht.Ausgeben (Wechselgeld);
          Ausgabeschacht.Ausgeben (Gewhlt);
        end;
        Gewhlt := Keine;
        Gtk.GEntry.Set_Text (Wahl, "");
      end if;
    end if;
  end Einwurf;

  procedure C_10 is new Einwurf (Cent_10);  -- Ausprgung kann nicht direkt
  procedure C_20 is new Einwurf (Cent_20);  -- als Rumpf verwendet werden
  procedure C_50 is new Einwurf (Cent_50);
  procedure E_1  is new Einwurf (Euro_1 );
  procedure E_2  is new Einwurf (Euro_2 );
  procedure E_5  is new Einwurf (Euro_5 );
  procedure E_10 is new Einwurf (Euro_10);

  procedure Cent_10 (Self: access Gtk.Button.Gtk_Button_Record'Class) renames C_10;  -- renaming
  procedure Cent_20 (Self: access Gtk.Button.Gtk_Button_Record'Class) renames C_20;  -- as body
  procedure Cent_50 (Self: access Gtk.Button.Gtk_Button_Record'Class) renames C_50;
  procedure Euro_1  (Self: access Gtk.Button.Gtk_Button_Record'Class) renames E_1 ;
  procedure Euro_2  (Self: access Gtk.Button.Gtk_Button_Record'Class) renames E_2 ;
  procedure Euro_5  (Self: access Gtk.Button.Gtk_Button_Record'Class) renames E_5 ;
  procedure Euro_10 (Self: access Gtk.Button.Gtk_Button_Record'Class) renames E_10;

  -- Abbruch

  procedure Abbruch (Grund: String) is
    Rckgeld: Natural := Waehrung.  -- Warum braucht es hier die Qualifizierung?
                         Betrag (Gezahlt);
  begin
    if Gewhlt /= Keine then
      Gewhlt := Keine;
      Gtk.GEntry.Set_Text (Wahl  , "");
      Gtk.GEntry.Set_Text (Betrag, "");
      Zeige (Titel     => "Abbruch " & Grund,
             Nachricht => (if Rckgeld = 0 then 60 * ' '  -- damit der Titel richtig dargestellt wird
                                           else "Entnehmen Sie das eingegebene Geld" & Image (Rckgeld)));
      Ausgabeschacht.Ausgeben (Gezahlt);
      Ausgabeschacht.Ausgeben (Gewhlt);
    end if;
  end Abbruch;

  procedure Abbruch (Self: access Gtk.Button.Gtk_Button_Record'Class) is
  begin
    Abbruch ("durch Kunden");
  end Abbruch;

  -- Revision

  procedure Leere_Speicherinhalt (Inhalt : in Stckelung) is
    -- Nur ein harmloser Mitarbeiter der Ada Verkehrsbetriebe
    -- leert den Automaten, um ihn vor Panzerknackern zu schtzen.
    -- Dazu wird der Automat ausgeschaltet und muss danach wieder neu
    -- gestartet werden.
    function Image (Stck: Geldstueck) return String is
    begin
      if Stck not in Akzeptiertes_Geldstck then
        return "";
      else
        return LF & Integer'Image (Inhalt (Stck)) & "  " & Geldstueck'Image (Stck) &
               Image (Akzeptiertes_Geldstck'Succ (Stck));
      end if;
    end Image;
  begin
    Zeige (Titel     => "Ada Verkehrsbetriebe",
           Nachricht => "Entleerung des Automaten:" & Image (Waehrung.Betrag (Inhalt)) & Image (Akzeptiertes_Geldstck'First));
  end Leere_Speicherinhalt;

  procedure Verkaufsstatistik (Verkauf: in Dagobert.Fahrkarten) is
    function Liste (Karte: Fahrkarte) return String is
    begin
      return LF & Integer'Image (Verkauf (Karte)) & "  " & Image (Karte) &
             (if Karte < Fahrkarte'Last then Liste (Fahrkarte'Succ (Karte)) else 30 * ' ');
    end Liste;
  begin
    Zeige (Titel     => "Ada Verkehrsbetriebe",
           Nachricht => "Verkaufsstatistik:" & Liste (Fahrkarte'First));
  end Verkaufsstatistik;

  procedure Revision (Self: access Gtk.Widget.Gtk_Widget_Record'Class) is
    Verkauf : Dagobert.Fahrkarten;
    Speicher: Stckelung;
  begin
    Abbruch ("wegen Revision");
    Dagobert.Panzerknacker (Verkauf, Speicher);
    Leere_Speicherinhalt   (Speicher);
    Verkaufsstatistik      (Verkauf);
  end;

  -- Bedienungsanleitung und Befrderungsbedingungen

  procedure Hilfe (Self: access Gtk.Button.Gtk_Button_Record'Class) is
  begin
    Zeige (Titel     => "Bedienungsanleitung",
           Nachricht => "Sie werden einen einfachen Automaten sehr glcklich machen," & LF &
                        "wenn Sie eine Fahrkarte bei mir bestellen." & LF &
                        "." & (41 * '=') & "." & LF &
                        "|   Jeder einzelne Tastendruck wird das Brutto-Sozial-Glck der   |" & LF &
                        "|  " & (28 * ' ') & "Ada Verkehrsbetriebe" & (29 * ' ') & "  |" & LF &
                        "|  " & (31 * ' ') & "erheblich erhhen." & (30 * ' ') & "  |" & LF &
                        "`" & (41 * '=') & "'");
    Zeige (Titel     => "Befrderungsbedingungen",
           Nachricht => "Die Karten berechtigen zu folgenden Fahrten:" & LF &
                        "*  Kurzstrecke: Einmalig durch ein Kapitel des Ada Referenzmanuals" & LF &
                        "*  Innenraum:  Eimalig durch den Sprachkern" & LF &
                        "*  Auߟenraum: Eimalig durch das gesamte Referenzmanual" & LF &
                        "*  Tageskarte: Lebenslang durch Ada-Land" & LF &
                        "Grenzberschreitungen in andere Lnder sind nur unter den in" & LF &
                        "Annex B aufgefhrten Bedingungen erlaubt." & LF &
                        "Grenzberschreitungen in alle anderen Lnder sind untersagt.");
  end Hilfe;

  -- Zeitberschreitung:
  -- Bei Kartenwahl wird der Timer gestartet.
  -- Bei jedem Geldstck wird der laufende Timer gelscht und ein neuer gestartet.
  -- Bei abgeschlossenem Kauf wird der Timer gelscht.
  -- Bei Abbruch wird der Timer ebenfalls gelscht.
  -- Bei Ablauf des Timers wird der Kauf abgebrochen.

  Timeout_Id:          Glib.Main.G_Source_Id;
  Timeout   : constant Glib.Guint := 10_000;  -- 10 s in ms

  function Zeitberschreitung (Button: Gtk.Button.Gtk_Button) return Boolean is
  begin
    --Ada.Text_IO.Put_Line ("Timeout" & Timeout_Id'Image);
    Abbruch ("wegen Zeitberschreitung");
    Stop_Timeout (Button);
    return True;
  end Zeitberschreitung;

  procedure Start_Timeout (Self: access Gtk.Button.Gtk_Button_Record'Class) is
    use Glib.Main;
  begin
    if Timeout_Id = 0 then
      Timeout_Id := Button_Timeout.Timeout_Add
        (Timeout, Zeitberschreitung'Access, Gtk.Button.Gtk_Button (Self));
      --Ada.Text_IO.Put_Line ("Start_Timeout" & Timeout_Id'Image);
    end if;
  end Start_Timeout;

  procedure Restart_Timeout (Self: access Gtk.Button.Gtk_Button_Record'Class) is
    use Glib.Main;
  begin
    if Timeout_Id /= 0 then
      Remove (Timeout_Id);
      Timeout_Id := Button_Timeout.Timeout_Add
        (Timeout, Zeitberschreitung'Access, Gtk.Button.Gtk_Button (Self));
      --Ada.Text_IO.Put_Line ("Restart_Timeout" & Timeout_Id'Image);
    end if;
  end Restart_Timeout;

  procedure Stop_Timeout (Self: access Gtk.Button.Gtk_Button_Record'Class) is
    use Glib.Main;
  begin
    if Timeout_Id /= 0 then
      --Ada.Text_IO.Put_Line ("Stop_Timeout" & Timeout_Id'Image);
      Remove (Timeout_Id);
      Timeout_Id := 0;
    end if;
  end Stop_Timeout;

end Callback;
