package body Fahrkartenautomat.Dagobert is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.1
  -- Datum     15 September 2021
  --====================================================================
  --
  --====================================================================
  -- Geschichte
  -- Autor  Version   Datum   Grund der nderung
  --  C.G.    1.1  15.09.2021 Kaufstatistik
  --  C.G.    1.0  08.09.2021 Anpassung der JEWL-Version 2.0
  --====================================================================

  Talerlein: Stckelung :=  -- frisch gefllte Maschine
    (Cent_10 => 20,         -- enthlt Geldstcke, damit
     Cent_20 => 10,         -- gewechselt werden kann;
     Cent_50 =>  5,         -- hier sehr knapp, sodass
     Euro_1  => 10,         -- Wechselprobleme bald
     Euro_2  =>  5,         -- auftauchen
     Euro_5  =>  5,
     Euro_10 =>  0);        -- als Wechelgeld unmglich

  -- Mit nchst kleinerer Mnze
  E5: constant Stckelung := (Euro_2  => 2, Euro_1  => 1, others => 0);
  E2: constant Stckelung := (Euro_1  => 2,               others => 0);
  E1: constant Stckelung := (Cent_50 => 2,               others => 0);
  C5: constant Stckelung := (Cent_20 => 2, Cent_10 => 1, others => 0);
  C2: constant Stckelung := (Cent_10 => 2,               others => 0);

  Geschft: Fahrkarten := (others => 0);

  --------------------------------------------------------------------------

  function "+" (L, R: Stckelung) return Stckelung
    with Inline
  is
  begin
    return Zwerg: Stckelung do
      for M in Zwerg'Range loop
        Zwerg (M) := L (M) + R (M);
      end loop;
    end return;
  end "+";

  function "*" (L: Natural; R: Stckelung) return Stckelung
    with Inline
  is
    Zwerg: Stckelung;
  begin
    for M in Zwerg'Range loop
      Zwerg (M) := L * R (M);
    end loop;
    return Zwerg;
  end "*";

  procedure Anpassen (M: in Akzeptiertes_Geldstck; Differenz: in Natural; Wechselgeld: in out Stckelung)
    with Inline
  is
    -- Teile fehlende Geldstcke in nchst kleinere Stcke
  begin
    Wechselgeld := (case M is
                      when Euro_5  => Wechselgeld + Differenz * E5,
                      when Euro_2  => Wechselgeld + Differenz * E2,
                      when Euro_1  => Wechselgeld + Differenz * E1,
                      when Cent_50 => Wechselgeld + Differenz * C5,
                      when Cent_20 => Wechselgeld + Differenz * C2,
                      when others  => Wechselgeld);
  end Anpassen;

  --------------------------------------------------------------------------

  procedure Speichere (Gewhlt       : in  Fahrkarte;
                       Zurckzuzahlen: in  Natural   ;
                       Gezahlt       : in  Stckelung;
                       Wechselgeld   : out Stckelung;
                       Wechselproblem: out Boolean   ) is
    Inhalt   : Stckelung := Talerlein + Gezahlt;
    Differenz: Natural;
  begin
    Wechselgeld := Stueckle (Zurckzuzahlen) (Akzeptiertes_Geldstck);
    for M in reverse Inhalt'Range loop
      if Wechselgeld (M) <= Inhalt (M) then
        Inhalt (M) := Inhalt (M) - Wechselgeld (M);  -- gengend Stcke M
      else
        if M = Cent_10 then
          Wechselproblem := True;
          Wechselgeld    := Gezahlt;  -- nicht genug Wechselgeld im Speicher
          return;
        end if;
        Differenz       := Wechselgeld (M) - Inhalt (M);  -- das fehlt
        Wechselgeld (M) := Inhalt (M);                    -- nimm soviel wie da ist
        Anpassen (M, Differenz, Wechselgeld);             -- und nimm fr den Rest kleinere Stcke
      end if;
    end loop;
    Wechselproblem     := False;
    Talerlein          := Inhalt;
    Geschft (Gewhlt) := @ + 1;
  end Speichere;

  procedure Panzerknacker (Verkauf : out Fahrkarten;
                           Speicher: out Stckelung) is
  begin
    Verkauf  := Geschft;
    Speicher := Talerlein;
  end Panzerknacker;

end Fahrkartenautomat.Dagobert;
