with Ada.Characters.Handling;
with Ada.Strings.UTF_Encoding.Strings;


with Glib.Main;
with Gtk.Handlers;
with Gtk.Window, Gtk.Widget;
with Gtk.Button, Gtk.GEntry;

with Fahrkartenautomat.Convert;

package body Fahrkartenautomat is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.3
  -- Datum     19 September 2021
  --====================================================================
  -- Obwohl String und UTF8_String Synonyme sind, verwende ich ersteren,
  -- wenn der Inhalt Latin_1, den letzteren, wenn der Inhalt zu UTF8
  -- konvertiert ist.
  --====================================================================
  -- Geschichte
  -- Autor  Version   Datum   Grund der nderung
  --  C.G.    1.3  19.09.2021 Abbruch bei Zeitberschreitung
  --  C.G.    1.2  15.09.2021 Callback fr Revision
  --  C.G.    1.1  12.09.2021 Character'Pos (164) als Euro-Platzhalter
  --  C.G.    1.0  11.09.2021
  --====================================================================

  Automat: Gtk.Window.Gtk_Window;

  Kurz, Innen, Gesamt, Tag: Gtk.Button.Gtk_Button;  -- Die Fahrkarten

  Hilfe: Gtk.Button.Gtk_Button;

  B_Cent_10, B_Cent_20, B_Cent_50,
  B_Euro_1, B_Euro_2, B_Euro_5, B_Euro_10: Gtk.Button.Gtk_Button;  -- Geldstcke zum Bezahlen

  Abbruch: Gtk.Button.Gtk_Button;  -- Doch nicht

  Wahl, Betrag: Gtk.GEntry.Gtk_GEntry;  -- Anzeige

  function "+" (Text: String) return Glib.UTF8_String renames Fahrkartenautomat.Convert;

  package Button_Timeout is new Glib.Main.Generic_Sources (Gtk.Button.Gtk_Button);
  package Button_Handler is new Gtk.Handlers.Callback     (Gtk.Button.Gtk_Button_Record);

  package Callback is

    procedure Karte    (Self: access Gtk.Button.Gtk_Button_Record'Class);

    procedure Cent_10  (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Cent_20  (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Cent_50  (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Euro_1   (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Euro_2   (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Euro_5   (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Euro_10  (Self: access Gtk.Button.Gtk_Button_Record'Class);

    procedure Abbruch  (Self: access Gtk.Button.Gtk_Button_Record'Class);

    procedure Revision (Self: access Gtk.Widget.Gtk_Widget_Record'Class);

    procedure Hilfe    (Self: access Gtk.Button.Gtk_Button_Record'Class);

    procedure Start_Timeout   (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Restart_Timeout (Self: access Gtk.Button.Gtk_Button_Record'Class);
    procedure Stop_Timeout    (Self: access Gtk.Button.Gtk_Button_Record'Class);

  end Callback;
  package body Callback is separate;

  function Image (Karte: Fahrkarte) return String is
    Image: constant String := Karte'Image;
  begin
    return Image (1) & Ada.Characters.Handling.To_Lower (Image (2 .. Image'Last));
  end Image;

  function Image (Geld: Integer) return String is
    Raw: constant String := Geld'Image;  -- Mindestlnge 3 (fr z.B. 10 Cent)
  begin
    if Geld <= 0 then
      return "";
    else      -- Vorkomma
      return (if Raw (Raw'Last - 2) = ' ' then " 0" else Raw (1 .. Raw'Last - 2)) & ',' &
              -- Nachkomma
             Raw (Raw'Last - 1 .. Raw'Last) & " ";
    end if;
  end Image;

  procedure Aufstellen is separate;

end Fahrkartenautomat;
