with Ada.IO_Exceptions;

separate (Fahrkartenautomat.GUI)
procedure Hilfe_Installieren (Hilfe: Button_Type) is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.2
  -- Datum     8 Juni 2021
  --====================================================================
  -- Das ist die Variante mit Tasking.
  -- Es gibt viele Methoden, Varianten zu implementieren.
  -- Dies ist eine davon.
  --
  -- Die Hilfe-Taste ist schon im Elterpaket GUI installiert; hier
  -- also null-Prozedur.
  -- In der Variante ohne Tasking wird die Taste verborgen.
  --
  -- Auch das Ada-Bild soll nur auf der erweiterten Version erscheinen.
  -- Aus Faulheit ist es hier eingefgt. So entstehen inkonsequente
  -- Strukturen, die im Lauf der Zeit Softwarepflege und -nderung
  -- enorm erschweren. Besser wr's also in diesem Fall, den Prozedur-
  -- namen passend zu ndern.
  -- Das Bild selbst stammt von Ms Christine M. Anderson, Ada 9X Project
  -- Office manager.
  -- Manchmal erscheint es nicht richtig beim Start; beim Manipulieren
  -- der Fenstergre erscheint es dann. (Keine Ahnung warum.)
  --====================================================================
  -- Geschichte
  -- Autor  Version  Datum    Grund der nderung
  --  C.G.    1.0  05.11.2020 Hilfe-Taste implementiert (hier null)
  --  C.G.    1.1  09.11.2020 Ada-Bild eingefgt
  --  C.G.    1.2  08.06.2021 Fehlerbehandlung, falls Bild fehlt
  --====================================================================

begin

  declare
    Ada_95    : constant Image_Type  := Image  (Name    => "Ada_9X.bmp");
    Ada_Canvas: constant Canvas_Type := Canvas (Parent  => GUI,
                                                Origin  => (X => 10, Y => 90),
                                                Width   => Width  (Ada_95),
                                                Height  => Height (Ada_95));
  begin
    Draw_Image (Canvas => Ada_Canvas,
                From   => (X => 0, Y => 0),
                Width   => Width  (Ada_95),
                Height  => Height (Ada_95),
                Image  => Ada_95);
  end;

exception
    -- Hier liegt ein grober Designfehler in JEWL vor.
    -- Die Funktion Image lst intern Name_Error aus,
    --    Ada.Streams.Stream_IO.Open (File, Name => Name, Mode => In_File);
    -- wenn Ada_9X.bmp nicht existiert, der jedoch am Ende der Funktion
    -- durch einen others-Behandler abgefangen wird. Dieser ruft Close auf
    -- fr eine folglich nicht existente Datei, daher hier Status_Error.
    -- Auch fr fehlerhaften Inhalt der Bilddatei werden alle Ausnahmen durch
    -- diesen Fehlerbehandler abgewrgt.
  when Ada.IO_Exceptions.Status_Error =>
    Show_Error ("Das Bild Ada_9X.bmp ist nicht vorhanden oder der Inhalt ist kaputt.");

end Hilfe_Installieren;
