with Fahrkartenautomat.GUI;

package body Fahrkartenautomat is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   2.1
  -- Datum     5 November 2020
  --====================================================================
  -- Implementierung mit JEWL ohne Tasking.
  --
  -- Probleme:
  -- * Bei Ausfhrung im DOS-Fenster werden die Umlaute nicht richtig
  --   angezeigt.
  --   (Im GPS im Run-Reiter ist die Anzeige richtig.)
  -- * Das Euro-Zeichen kann nicht verwendet werden.
  -- * Abbruch bei Zeitberschreitung kann nur mit Tasking gelst werden
  --====================================================================
  -- Geschichte
  -- Autor  Version  Datum    Grund der nderung
  --  C.G.    1.0  12.04.2014 Created for Ada tasking course
  --  C.G.    2.0  03.11.2020 Funktional
  --  C.G.    2.1  05.11.2020 Neues Kommando Hilfe
  --====================================================================

  use GUI;

  -- Zustand des Automaten

  Automat_Frei  : Boolean := True;
  Gewhlte_Karte: Fahrkarte;
  Zu_Zahlen     : Integer;  -- kann negativ werden bei berzahlung

  Eingegeben: Stckelung := (others => 0);

  procedure Whle_Fahrkarte (Kommando: in Kartenwahl) is
  begin
    case Kommando is
      when Kurzstrecke  => Gewhlte_Karte := Kurzstrecke;
      when Innenraum    => Gewhlte_Karte := Innenraum;
      when Gesamtgebiet => Gewhlte_Karte := Gesamtgebiet;
      when Tageskarte   => Gewhlte_Karte := Tageskarte;
    end case;
    Automat_Frei := False;
    Eingegeben   := (others => 0);
    Zu_Zahlen    := Preis (Gewhlte_Karte);
    Anzeige_gewhlte_Karte (Gewhlte_Karte'Image);
    Anzeige_Zu_Zahlen      (Zu_Zahlen);
  end Whle_Fahrkarte;

  procedure Prfe_Summe (Kommando: in Bezahlen; Ergebnis: out Ausgabe) is
    Stck: constant Geldstueck := Geldstueck'Val (GUI_Kommando'Pos (Kommando) - 2);
  begin
    Eingegeben (Stck) := Eingegeben (Stck) + 1;
    Zu_Zahlen          := Zu_Zahlen - Wert (Stck);
    Anzeige_Zu_Zahlen (Zu_Zahlen);
    if Zu_Zahlen <= 0 then
      Anzeige_Ausgabe        (abs Zu_Zahlen);
      Anzeige_gewhlte_Karte ("");
      Ergebnis     := (Gewhlte_Karte, Stueckle (abs Zu_Zahlen) (Akzeptiertes_Geldstck));
      Automat_Frei := True;
      Eingegeben   := (others => 0);
    end if;
  end Prfe_Summe;

  procedure Abbruch (Ergebnis: out Ausgabe; Grund: in String) is
  begin
    Anzeige_gewhlte_Karte ("");                          -- Reihenfolge wichtig!
    Anzeige_Zu_Zahlen      ( 0);                          -- Erst Anzeige anpassen, dann
    Anzeige_Abbruch        (Betrag (Eingegeben), Grund);  -- warten auf Eingabe.
    Automat_Frei := True;
    Ergebnis     := (Keine, Eingegeben);
    Eingegeben   := (others => 0);
  end Abbruch;

  -----------------------------------------------------------------------------

  procedure Kaufe_Karte (Ergebnis: out Ausgabe; Stop: out Boolean) is
    -- Da Tasten etc. nicht stillgelegt werden knnen (oder knnen sie?),
    -- werden sie nur ausgewertet, wenn der Automat im passenden Zustand ist.
    -- (Tasten knnen versteckt werden.)
    Kommando: GUI_Kommando;
  begin
    loop
      Kommando := Nchstes_Kommando;
      case Kommando is
        when Kartenwahl => if Automat_Frei then
                             Whle_Fahrkarte (Kommando);
                           else
                             null;  -- Karte schon gewhlt
                           end if;
        when Bezahlen   => if not Automat_Frei then
                             Prfe_Summe (Kommando, Ergebnis);
                             exit when Automat_Frei;
                           else
                             null;  -- noch keine Karte gewhlt
                           end if;
        when Abbruch    => if not Automat_Frei then
                             Abbruch (Ergebnis, "durch Kunden");
                             exit;
                           else
                             null;  -- nichts abzubrechen
                           end if;
        when Ende       => Ergebnis := (Keine, Eingegeben);  -- kann immer passieren
                           exit;
        when others     => null;  -- nicht implementiert in dieser Version
                                  -- Sollten neue GUI-Komandos implementiert werden,
                                  -- bleibt mit der others-Wahl diese Variant unverndert.
      end case;
    end loop;
    Stop := Kommando = Ende;
  end Kaufe_Karte;

end Fahrkartenautomat;
