with Ada.Characters.Latin_1;
with Ada.Strings.Fixed;

with Waehrung;
with JEWL.Windows;

package body Fahrkartenautomat.GUI is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   2.3
  -- Datum     29 Mrz 2023
  --====================================================================
  --
  --====================================================================
  -- Geschichte
  -- Autor  Version  Datum    Grund der nderung
  --  C.G.    2.3  29.03.2023 Kartenwahl ausgrauen,
  --                          Geldtasten unsichtbar
  --  C.G.    2.2  23.03.2023 Verkaufsstatistik neu (nach GtkAda-Lsung)
  --  C.G.    2.1  06.11.2020 Hilfe-Taste ist neu
  --  C.G.    2.0  03.11.2020 Neue Implementierung mit JEWL
  --  C.G.    1.0  12.04.2014 Created with Text_IO for Ada tasking
  --                          course
  --====================================================================

  LF: Character renames Ada.Characters.Latin_1.LF;  -- funktioniert in JEWL, aber undokumentiert

  function Image (Geld: Integer) return String is
    Raw: constant String := Geld'Image;  -- Mindestlnge 3 (fr z.B. 10 Cent)
  begin
    if Geld <= 0 then
      return "";
    else      -- Vorkomma
      return (if Raw (Raw'Last - 2) = ' ' then " 0" else Raw (1 .. Raw'Last - 2)) & ',' &
              -- Nachkomma
                 Raw (Raw'Last - 1 .. Raw'Last);
    end if;
  end Image;

  -- Global bentigte Anzeigen

  package GUI_Fenster is new JEWL.Windows (GUI_Kommando);
  use GUI_Fenster;
  GUI   : Frame_Type := Frame (Width   => 580, Height => 350,    -- der gesamte Rahmen
                               Title   => "Fahrkartenautomat",
                               Command => Ende);
  Ticket: Label_Type := Label (Parent  => GUI,                   -- Anzeige der gewhlten Karte
                               Origin  => (X => 310, Y => 100),
                               Width   => 90, Height => 25,
                               Text    => "",
                               Align   => Right);
  Rest  : Label_Type := Label (Parent  => GUI,                   -- Anzeige des noch zu zahlenden Betrags
                               Origin  => (X => 310, Y => 130),
                               Width   => 90, Height => 25,
                               Text    => "",
                               Align   => Right);
  Karten_Taste : array (Kartenwahl) of Button_Type;
  Geld_Taste   : array (Bezahlen  ) of Button_Type;
  Abbruch_Taste: Button_Type;

  procedure Hilfe_Installieren (Hilfe: Button_Type) is separate;

  procedure Initialisieren is
    -- Enthlt alle Teile, die nicht global bentigt werden.
    Hilfe_Taste  : Button_Type := Button (Parent  => GUI,
                                          Origin  => (X => 430, Y => 100),
                                          Width   => 50, Height => 50,
                                          Text    => "?",
                                          Font    => Font ("Arial", Size => 30, Bold => True),
                                          Command => Hilfe);
    Karten_Preis: array (Kartenwahl) of Label_Type;
    Gewhlt     : Label_Type := Label (Parent  => GUI,
                                       Origin  => (X => 180, Y => 100),
                                       Width   => 120, Height => 25,
                                       Text    => "Gewhlte Karte:",
                                       Align   => Left);
    Zu_Zahlen   : Label_Type := Label (Parent  => GUI,
                                       Origin  => (X => 180, Y => 130),
                                       Width   => 120, Height => 25,
                                       Text    => "Zu zahlender Betrag:",
                                       Align   => Left);
  begin
    Hilfe_Installieren (Hilfe_Taste);
    -- Alle Tastengruppen zentriert
    for K in Karten_Taste'Range loop
      Karten_Taste (K) := Button (Parent  => GUI,
                                  Origin  => (X => -130 + Kartenwahl'Pos (K) * 140, Y => 30),
                                  Width   => 120, Height => 25,
                                  Text    => K'Image,
                                  Command => K);
      Karten_Preis (K) := Label  (Parent  => GUI,
                                  Origin  => (X => -130 + Kartenwahl'Pos (K) * 140, Y => 60),
                                  Width   => 120, Height => 20,
                                  Text    => Image (Preis (Fahrkarte'Val (Kartenwahl'Pos (K)))),
                                  Align   => Centre);
    end loop;
    for G in Geld_Taste'Range loop
      Geld_Taste (G) := Button (Parent  => GUI,
                                Origin  => (X => (if G < Euro_1 then -100 else -310) + Bezahlen'Pos (G) * 60,
                                            Y => (if G < Euro_1 then  160 else  190)),
                                Width   => 40, Height => 25,
                                Text    => Image (Wert (Geldstueck'Val (Bezahlen'Pos (G) - 2))),
                                Command => G);
    end loop;
    Abbruch_Taste := Button (Parent  => GUI,
                             Origin  => (X => 250, Y => 260),
                             Width   => 80, Height => 25,
                             Text    => "Abbruch",
                             Command => Abbruch);
  end Initialisieren;

  function Nchstes_Kommando return GUI_Kommando renames GUI_Fenster.Next_Command;

  procedure Bedienungsanleitung is
    function "*" (N: Natural; C: Character) return String renames Ada.Strings.Fixed."*";
  begin
    Show_Message (Title => "Bedienungsanleitung",
                  Text  => "Sie werden einen einfachen Automaten sehr glcklich machen," & LF &
                           "wenn Sie eine Fahrkarte bei mir bestellen." & LF &
                           "." & (41 * '=') & "." & LF &
                           "|  Jeder einzelne Tastendruck wird das  Brutto-Sozial-Glck der  |" & LF &
                           "|  " & (34 * ' ') & "Ada Verkehrsbetriebe" & (34 * ' ') & "  |" & LF &
                           "|  " & (36 * ' ') & "erheblich erhhen." & (36 * ' ') & "  |" & LF &
                           "`" & (41 * '=') & "'");
    Show_Message (Title => "Befrderungsbedingungen",
                  Text  => "Die Karten berechtigen zu folgenden Fahrten:" & LF &
                           "*  Kurzstrecke: Einmalig durch ein Kapitel des Ada Referenzmanuals" & LF &
                           "*  Innenraum: Eimalig durch den Sprachkern" & LF &
                           "*  Auenraum: Eimalig durch das gesamte Referenzmanual" & LF &
                           "*  Tageskarte: Lebenslang durch Ada-Land" & LF &
                           "Grenzberschreitungen in andere Lnder sind nur unter den in" & LF &
                           "Annex B aufgefhrten Bedingungen erlaubt." & LF &
                           "Grenzberschreitungen in alle anderen Lnder sind untersagt.");
  end Bedienungsanleitung;

  procedure Anzeige_gewhlte_Karte (Karte: in String) is
  begin
    Set_Text (Ticket, Karte);
  end Anzeige_gewhlte_Karte;

  procedure Anzeige_Zu_Zahlen (Geld: in Integer) is
  begin
    Set_Text (Rest, Image (Geld));
  end Anzeige_Zu_Zahlen;

  procedure Anzeige_Ausgabe (Wechselgeld: in Natural) is
  begin
    Show_Message (Title => "Fahrkartenausgabe",
                  Text  => "Entnehmen Sie die Fahrkarte" &
                           (if Wechselgeld = 0 then ""
                                               else " und das Wechselgeld" & Image (Wechselgeld)));
  end Anzeige_Ausgabe;

  procedure Anzeige_Abbruch (Rckgeld: in Natural; Grund: in String) is
  begin
    Show_Message (Title => "Abbruch " & Grund,
                  Text  => (if Rckgeld = 0 then ""
                            else "Entnehmen Sie das eingegebene Geld" & Image (Rckgeld)));
  end Anzeige_Abbruch;

  procedure Anzeige_Speicherinhalt (Inhalt: in Stckelung) is
    -- Nur ein harmloser Mitarbeiter der Ada Verkehrsbetriebe
    -- leert den Automaten, um ihn vor Panzerknackern zu schtzen.
    -- Dazu wird der Automat ausgeschaltet und muss danach wieder neu
    -- gestartet werden.
    use Waehrung;
    function Image (Stck: Geldstueck) return String is
    begin
      if Stck not in Akzeptiertes_Geldstck then
        return "";
      else
        return LF & Integer'Image (Inhalt (Stck)) & "  " & Geldstueck'Image (Stck) &
                    Image (Akzeptiertes_Geldstck'Succ (Stck));
      end if;
    end Image;
  begin
    Show_Message (Title => "Ada Verkehrsbetriebe",
                  Text  => "Entleerung des Automaten:" & Image (Betrag (Inhalt)) & Image (Akzeptiertes_Geldstck'First));
  end Anzeige_Speicherinhalt;

  procedure Anzeige_Verkaufsstatistik (Verkauf: in Verkaufsstatistik) is
    -- Die Ada Verkehrsbetriebe sind natrlich auch an den Verkaufszahlen interssiert.
    -- Der Mitarbeiter bringt auch einen Beleg dazu mit.
    function Liste (Karte: Fahrkarte) return String is
    begin
      return LF & Integer'Image (Verkauf (Karte)) & "  " & Fahrkarte'Image (Karte) &
             (if Karte < Fahrkarte'Last then Liste (Fahrkarte'Succ (Karte)) else "");
    end Liste;
  begin
    Show_Message (Title => "Ada Verkehrsbetriebe",
                  Text  => "Verkaufsstatistik:" & Liste (Fahrkarte'First));
  end Anzeige_Verkaufsstatistik;

  procedure Aktiviere_Karten_Tasten (Aktiv: in Boolean) is
  begin
    for Karte of Karten_Taste loop
      Enable (Control => Karte, Enabled => Aktiv);
    end loop;
  end Aktiviere_Karten_Tasten;

  function Karten_Tasten_Aktiv return Boolean is
  begin
    return Enabled (Karten_Taste (Karten_Taste'First));  -- irgendeine
  end Karten_Tasten_Aktiv;

  procedure Zeige_Geld_Tasten (Sichtbar: in Boolean) is
  begin
    for Geld of Geld_Taste loop
      Show (Window => Geld, Visible => Sichtbar);
    end loop;
    Show (Abbruch_Taste, Sichtbar);
  end Zeige_Geld_Tasten;

  function Geld_Tasten_Sichtbar return Boolean is
  begin
    return Visible (Abbruch_Taste);  -- irgendeine
  end Geld_Tasten_Sichtbar;

begin

  Initialisieren;

end Fahrkartenautomat.GUI;
