package body Sim_Pools is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      17 January 2018
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  17.01.2018 Created for Ada course
  --====================================================================

  overriding
  procedure Allocate (Pool                    : in out Sim_Pool;
                      Storage_Address         :    out System.Address;
                      Size_in_Storage_Elements: in     Storage_Count;
                      Alignment               : in     Storage_Count) is
    Index: Storage_Count := Pool.Free;
  begin
    while Pool.Store (Index)'Address mod Alignment /= 0 loop
      Index := 1 + Index;
    end loop;
    Storage_Address := Pool.Store (Index)'Address;
    Pool.Free       := Index + Size_in_Storage_Elements;
  end Allocate;

  overriding
  function Storage_Size (Pool: Sim_Pool) return System.Storage_Elements.Storage_Count is
  begin
    return Pool.Free - 1;
  end Storage_Size;

  not overriding
  function Address (Pool: Sim_Pool) return System.Address is
  begin
    return Pool.Store'Address;
  end Address;

  not overriding
  procedure Copy_From (Pool   : out Sim_Pool;
                       Address: in  System.Address) is
    -- No time limit on this operation.
    Buffer: System.Storage_Elements.Storage_Array (1 .. Pool.Size);
    for Buffer'Address use Address;
    pragma Import (Convention => C, Entity => Buffer);
  begin
    Pool.Store := Buffer;
  end Copy_From;

end Sim_Pools;
