with System.Storage_Pools,
     System.Storage_Elements;

package Sim_Pools is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      17 January 2018
  --====================================================================
  -- This package provides the Sim_Pool type to be used for snapshots.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  17.01.2018 Created for Ada course
  --====================================================================

  subtype Storage_Count is System.Storage_Elements.Storage_Count;

  type Sim_Pool (Size: Storage_Count) is new System.Storage_Pools.Root_Storage_Pool with private;

  overriding
  procedure Allocate (Pool                    : in out Sim_Pool;
                      Storage_Address         :    out System.Address;
                      Size_in_Storage_Elements: in     Storage_Count;
                      Alignment               : in     Storage_Count);
  -- Allocate will be called for each allocation ("new XXX") using this Pool.
  -- See RM 13.11.

  overriding
  function Storage_Size (Pool: Sim_Pool) return Storage_Count;
  -- returns the current fill level.

  not overriding
  function Address (Pool: Sim_Pool) return System.Address;
  -- returns the address of the Pool's storage.

  not overriding
  procedure Copy_From (Pool   : out Sim_Pool;
                       Address: in  System.Address);
  -- takes the data beginning at Address and stores Pool.Size storage elements
  -- in Pool.

private

  use type System.Storage_Elements.Storage_Offset;

  type Sim_Pool (Size: Storage_Count) is new System.Storage_Pools.Root_Storage_Pool with record
    Store: System.Storage_Elements.Storage_Array (1 .. Size);
    Free : System.Storage_Elements.Storage_Count := 1;
  end record;

  -- Will never be called (no Unchecked_Deallocation) for Sim_Pool.
  overriding
  procedure Deallocate (Pool                    : in out Sim_Pool;
                        Storage_Address         : in     System.Address;
                        Size_In_Storage_Elements: in     Storage_Count;
                        Alignment               : in     Storage_Count) is null;

end Sim_Pools;
