with Ada.Text_IO;

package body Array_Text_IO is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      19 February 2023
  --====================================================================
  -- Correct the specification accordingly!
  --
  -- See the following design decision at work:
  -- Text_IO was designed for file operations with an important feature:
  -- If your program writes into a file, you can take the sequence of
  -- code that writes the file, replace every Put_* operation by the
  -- symmetric Get_* operation, and you will read the file without
  -- error.
  --
  -- Possible further formats:
  --    (a, b, c, ...)           positional
  --    (i => a, i+1 => b, ...)  named
  -- What about input checks?
  -- * Syntax errors
  -- * Number of components in accordance with bounds
  -- * Indices in ascending order
  -- * etc
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.1  19.02.2023 Corrected RM reference
  --  C.G.    1.0  25.09.2020 Created for Ada course
  --====================================================================

  -- Solve this problem of RM A.10.10(17/1):
  package Index_Text_IO is new Ada.Text_IO.Enumeration_IO (Index);  -- ?
  use Index_Text_IO;

  procedure Put (Item: in Array_Type) is
  begin
    Put (Item'First);
    Put (Item'Last );
    for Comp of Item loop  -- Ada 2012
      Put (Comp);
    end loop;
  end Put;

  --  procedure Get (Item: out Array_Type) is
  --  begin
  --    ??
  --  end Get;

  function Get return Array_Type is
    First, Last: Index;
  begin
    Get (First);
    Get (Last );
    return Item: Array_Type (First .. Last) do
      for Comp of Item loop
        Get (Comp);
      end loop;
    end return;
  end Get;

end Array_Text_IO;
