with Ada.Text_IO;
use  Ada.Text_IO;
with Ada.Numerics.Elementary_Functions;
use  Ada.Numerics.Elementary_Functions;

procedure E40 is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      28 Mai 2023
  --====================================================================
  -- Project Euler - Problem 40 -- Champernowne's Constant
  -- https://projecteuler.net/problem=40
  --====================================================================
  -- History
  -- Author Version   Date    Grund der nderung
  --  C.G.    1.0  28.05.2023 Erzeugt fr Ada-Kurs
  --====================================================================

  -- Which is the position I of the first digit occupied by N in this sequence?
  --
  -- 0.123456789101112131415161718192021...
  --              ^ The 12th digit is 1.
  --
  --                 |           N            |       Ith digit                      | I of first N  | I of last N |
  -- 1 digit numbers |    1      ..   9       |        1 .. 9                        |        1      |        9    |
  -- 2 digit numbers |   10      ..  99       |      9       + 1 +   2  *(N -  10)   |       10      |      188    |
  -- 3 digit numbers |  100      .. 999       |    188       + 2 +   3  *(N - 100)   |      190      |     2887    |
  -- k digit numbers | 10**(k-1) .. 10**k - 1 | I(10**L - 1) + L + (L+1)*(N - 10**L) |     2890      |    38886    |
  --                                              where L = Floor (lg N) = k - 1     |    38890      |   488885    |
  --
  -- What we really want to know: Which is the Ith digits in this number, I > 0?
  ------------------------------------------------------------------------------------------------------------------
  -- Computers invade our lives. They spy on us. They listen to our private
  -- conversations - Siri etc. We have to defend our privacy.
  --
  -- How?
  --
  -- Give them work to do! Do not optimize the code. Try to make it as slow as
  -- possible, but do not insert junk - just make the algorithm complicated.
  ------------------------------------------------------------------------------

  function I (N: Positive) return Positive is
    function L return Natural is (Natural (Float'Floor (Log (Float (N), Base => 10.0))));
  begin
    if L = 0 then
      return N;
    else
      return I (10**L - 1) + L + (L+1)*(N - 10**L);
    end if;
  end I;

  function N (J: Positive) return Positive is  -- inverse of I(N)
    L: Natural := 0;
  begin
    loop
      exit when J in I (10**L) .. I (10**(L+1)) - 1;
      L := L + 1;
    end loop;
    return (J - I (10**L)) / (L + 1) + 10**L;
  end N;

  function D (J: Positive) return Natural is   -- J = 10**4 => N = 2777 => I = 9998 = J-2
    LG: constant Natural  := Natural (Float'Floor (Log (Float (N (J)), Base => 10.0)));
  begin
    return (N (J) / 10**(LG - (J - I (N (J))))) rem 10;  -- 2777
  end D;                                                 --   ^

begin

  Put_Line ("Answer =" & Integer'Image ([for i in 0 .. 6 => D (10**i)]'Reduce ("*", 1)));

end E40;
