package Preelaboration_Surprise is

  pragma Preelaborate;

  generic
    type T is private;
    Default: T;
  package GP is
  end GP;

  type Screen is record
    X, Y: Positive;
  end record;

  Upper_Left: constant Screen := (others => 1);

  -- Surprise:
  -- This is a problem with pragma Preelaborate and the
  -- definition of Static in Ada.
  -- To paraphrase Randy Brukardt:
  --   Don't confuse the concept of elaboration with execution of explicit
  --   elaboration code.
  --   Everything in Ada is elaborated at some point. If you look around
  --   the Standard, you'll see lots of statements like "The elaboration
  --   of <blah> has no effect.". <blah> is elaborated, but that elaboration
  --   does nothing.
  --   And Ada compilers don't generate code to do elaboration unless they
  --   have to. So this affects only a small percentage of code, which is
  --   executed only once. It's the least likely thing to matter in a
  --   program.
  --   There's little value to pragma Preelaborate. Compilers will almost
  --   always generate the best possible elaboration code whether or not
  --   that pragma is applied. And regardless of C.4, not all preelaborated
  --   code can be done before load time, so it isn't providing a benefit
  --   in forcing that sort of code, either.
  --   It's one of the ideas where Ada tried to accomplish something, but
  --   didn't quite succeed. Pure is like that as well, but at least it
  --   provides some benefits for Annex E code. Preelaborate doesn't even
  --   do that.
  --   Performance isn't likely to be an issue (the elaboration code, if any,
  --   only runs once). It would take a pretty contrived program to notice
  --   the cost.
  --   So, to me, preelaborate is not solving any problems other than getting
  --   rid of a few elaboration checks. And GNAT has found a much better way
  --   to do that. So, in particular, if you are using GNAT, preelaborate
  --   buys you little other than error messages.
  --           Randy.

  package Bad  is new GP (Screen, Default => Upper_Left);
  -- 49:46 non-static constant in preelaborated unit
  -- 49:46 static expression must have scalar or string type (RM 4.9(2))

  package Good is new GP (Screen, Default => (others => 1));  --OK

end Preelaboration_Surprise;
