with Ada.Text_IO;
use  Ada.Text_IO;
with Ada.Assertions;
use  Ada.Assertions;

procedure Sliding is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   2.0
  -- Date      11 June 2023
  --====================================================================
  -- Show sliding parameter with subtype conversion.
  -- Sliding occurs only for subtype conversion to constrained subtypes.
  -- String_5 is unconstrained notwithstanding the predicate.
  -- Slide_to_5 is constrained.
  -- Run this with both assertion policies Ignore and Check.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    2.0  11.06.2023 Ada 2022: renaming of view conversion
  --  C.G.    1.1  15.04.2020 Improved for public version
  --  C.G.    1.0  07.03.2018 Created for Ada course
  --====================================================================

--pragma Assertion_Policy (Ignore);
  pragma Assertion_Policy (Check);

  subtype String_1_3 is String (1 .. 3);

  procedure P1 (X: in out String) is
  begin
    Put (X'First'Image & X'Last'Image & ' ');
    X (X'First) := 'a';
    X (X'Last)  := 'A';
  end P1;

  procedure P2 (X: in out String_1_3) is
  begin
    Put (X'First'Image & X'Last'Image & ' ');
    X (X'First) := 'A';
    X (X'Last)  := 'a';
  end P2;

  Text: String := "Lady Ada";  -- bounds 1 and 8
  T13 : String renames String_1_3 (Text (6 .. 8));  -- Ada 2022; view conversion is only readable

  -----------------

  subtype String_5 is String with Dynamic_Predicate => String_5'First = 5;

  procedure P (X: String_5) is
  begin
    Put (X'First'Image);
  end P;

  procedure Q (X: String) is
    subtype Slide_to_5 is String (5 .. 5 + X'Length - 1);
    procedure Call_P_Sliding (X: Slide_to_5) is
    begin
      P (X);
    end Call_P_Sliding;
    procedure Call_P_No_Sliding (X: String_5) is
    begin
      P (X);
    end Call_P_No_Sliding;
  begin
    Call_P_Sliding    (X);  -- sliding to 5 .. something
    Call_P_No_Sliding (X);  -- predicate check that X'First = 5
    Put_Line (" Either assertion policy is Ignore or compiler bug");
    -- Note: GNAT CE 2018 does the predicate check and raises the exception,
    --       GNAT CE 2019 does not. The problem has been reported to Ada Core.
    --       GNAT CE 2021 Problem corrected.
  exception
    when Assertion_Error => Put_Line (" Assertion_Error raised as expected");
  end Q;

  S1: constant String := "";
  S2: constant String (-100 .. -10_000) := "";
  S3: constant String := "Lady Ada";
  S4: constant String (101 .. 101 + S3'Length - 1) := S3;

begin

  P1 (            Text (6 .. 8) );  Put_Line (Text);  -- " 6 8 Lady adA"
  P2 (            Text (6 .. 8) );  Put_Line (Text);  -- " 1 3 Lady Ada"  implicit sliding with subtype conversion
  P1 (String_1_3 (Text (6 .. 8)));  Put_Line (Text);  -- " 1 3 Lady adA"  explicit sliding with view conversion

  Put (T13'First'Image & T13'Last'Image & ' ');  Put_Line (T13);  -- " 1 3 adA"  Ada 2022
  -- T13 := "ill";  -- illegal, left hand side of assignment must be a variable
  -- P1  (T13);  Put_Line (T13);  -- illegal: actual for X
  -- P2  (T13);  Put_Line (T13);  -- must be a variable

  -----------------

  Q (S1);
  Q (S2);
  Q (S3);
  Q (S4);

end Sliding;

