-- This test program reads an entire text file into a variable
-- of type Text_Container which is a vector of unbounded strings.
-- The memory is automatically freed when the procedure is left.
--
-- No pointers needed, no manual reclamation, no garbage waiting
-- to be collected!

with Ada.Text_IO.Unbounded_IO;
use  Ada.Text_IO, Ada.Text_IO.Unbounded_IO;
with Ada.Strings.Unbounded;
use  Ada.Strings.Unbounded;
with Ada.Containers.Vectors;

procedure Text_Container_Example is

  package Text_Containers is new Ada.Containers.Vectors (Positive, Unbounded_String);

  subtype Text_Container is Text_Containers.Vector;

  Name: constant String := "text_container_example.adb";

  Text: Text_Container;
  File: File_Type;

begin

  -- Read the original

  Open (File, In_File, Name);

  while not End_of_File (File) loop
    Text.Append (Get_Line (File));
  end loop;

  Close (File);

  -- Produce a copy

  Create (File, Out_File, "copy_of_" & Name);

  for Line of Text loop
    Put_Line (File, Line);
  end loop;

  Close (File);

end Text_Container_Example;