with Ada.Text_IO, Ada.Integer_Text_IO;

procedure Wochentage is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      23 November 2020
  --====================================================================
  -- Nur eine Skizze.
  -- Datum im Format dd.mm.yyyy
  --
  -- Offensichtlich gibt es Uneinigkeit ber die genaue Herkunft der
  -- Formeln. Man findet die untere Formel auch als von Zeller stammend.
  --
  -- In Wikipedia findet man Zellers Kongruenz als
  -- Wochentag = (13*(Monat+1)/5 + Tag + Jahr + Jahr/4 +
  --              Jahrhundert/4  2*Jahrhundert) mod 7
  -- wobei Samstag = 0 ist; die Monate Januar und Februar gelten als
  -- 13. und 14. Monat des Vorjahres.
  --
  -- Die Formel hier stammt nach Wikipedia vom sterreichischen
  -- Mathematiker Georg Glaeser, geb. 1955.
  -- Gregorianisches Datum:
  -- Wochentag = ((26*Monat-2)/10 + Tag + Jahr + Jahr/4 +
  --              Jahrhundert/4  2*Jahrhundert) mod 7
  -- Julianisches Datum:
  -- Wochentag = ((26*Monat-2)/10 + Tag + Jahr + Jahr/4 +
  --              5  Jahrhundert) mod 7
  -- wobei Sonntag = 0 und Januar und Februar als 11. und 12. Monat des
  -- Vorjahres gelten und Mrz der (historisch richtige) erste Monat des
  -- Jahres ist.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  16.10.2015 Created for Ada course
  --  C.G.    1.1  23.11.2020 Korrektur: Ursprung der Formel
  --====================================================================

  subtype Eingabe is String (1 .. 11);
  Zeile: Eingabe;
  Last : Natural;

  subtype Tag_Bereich   is Integer range 1 ..  2;
  subtype Monat_Bereich is Integer range 4 ..  5;
  subtype Jahr_Bereich  is Integer range 7 .. 10;

  Tag, Monat, Jahr, Jahrhundert: Positive;
  Wochentag: Natural range 0 .. 6;

begin

  Ada.Text_IO.Get_Line (Zeile, Last);  -- Datum einlesen vom Terminal

  Ada.Integer_Text_IO.Get (Zeile (Tag_Bereich  ), Tag  , Last);  -- Teile auswerten
  Ada.Integer_Text_IO.Get (Zeile (Monat_Bereich), Monat, Last);  -- mit Get, das aus
  Ada.Integer_Text_IO.Get (Zeile (Jahr_Bereich ), Jahr , Last);  -- String liest.

  -- Eingaben auf Fehler berprfen (Ausnahmebehandlung?)

  ...

  -- Formel anwenden

  Wochentag := bla;  -- Zellers Kongruenz

end Wochentage;
