with Stack;
with Ada.Text_IO;
use  Ada.Text_IO;

function Infix_to_Postfix (X: String) return String is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      28 January 2021
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  28.01.2021 Created for Ada course
  --====================================================================

  subtype Operand  is Character range 'A' .. 'Z';
  subtype Operator is Character with Static_Predicate => Operator in '-' | '+' | '*' | '/' | '^';
  subtype Extended is Character with Static_Predicate => Extended in Operator | '(' ;

  type Priority is range 0 .. 3;

  function Priority_of (Op: Extended) return Priority is
    (case Op is
       when '('       => 0,
       when '-' | '+' => 1,
       when '*' | '/' => 2,
       when '^'       => 3);

  Infix  : constant String := '(' & X & ')';
  Postfix: String (1 .. X'Length);
  Filled : Natural := 0;

begin

  for C of Infix loop
    case C is
      when Operand     => Filled := Filled + 1;
                          Postfix (Filled) := C;
      when Operator    => while Priority_of (C) <= Priority_of (Stack.Top) loop
                            Filled := Filled + 1;
                            Postfix (Filled) := Stack.Top;
                            Stack.Pop;
                          end loop;
                          Stack.Push (C);
      when '('         => Stack.Push (C);
      when ')'         => while Stack.Top in Operator loop
                            Filled := Filled + 1;
                            Postfix (Filled) := Stack.Top;
                            Stack.Pop;
                          end loop;
                          Stack.Pop;
      when others      => raise Program_Error;
    end case;
  end loop;

  return Postfix (1 .. Filled);

end Infix_to_Postfix;
