with Ada.Text_IO;

package body UPN is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      30 January 2021
  --====================================================================
  -- Algorithm:
  -- # Remove from Ada expression spaces and replace ** by ^.
  -- # Transform this Infix to Postfix.
  -- # Replace operands by values and evaluate the Postfix.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  30.01.2021 Created for Ada course
  --====================================================================

  function Infix_to_Postfix (X: String) return String is separate;

  function Scan (Expression: String) return String is
    -- Remove from Ada expression spaces and replace ** by ^.
    Infix : String (1 .. Expression'Length);
    Filled: Natural := 0;
    Skip  : Boolean := False;
  begin
    for C in Expression'Range loop
      if Skip or Expression (C) = ' ' then
        Skip := False;
        goto Next;  -- space to be skipped or * already scanned
      end if;
      Filled := Filled + 1;
      if Expression (C) = '*' and then Expression (C+1) = '*' then  -- C cannot be Last
        Skip:= True;
        Infix (Filled) := '^';
      else
        Infix (Filled) := Expression (C);
      end if;
      <<Next>>
    end loop;
    return Infix (1 .. Filled);
  end Scan;

  function Process (Postfix: String; Coefficients: Values) return Integer is
    -- Replace operands by values and evaluate the Postfix.
    High   : Natural := 0;
    Content: array (1 .. 10) of Integer;
  begin
    for C of Postfix loop
      case C is
        when Operand => High := High + 1;
                        Content (High) := Coefficients (C);
        when '+'     => High := High - 1;
                        Content (High) := Content (High) + Content (High+1);
        when '-'     => High := High - 1;
                        Content (High) := Content (High) - Content (High+1);
        when '*'     => High := High - 1;
                        Content (High) := Content (High) * Content (High+1);
        when '/'     => High := High - 1;
                        Content (High) := Content (High) / Content (High+1);
        when '^'     => High := High - 1;
                        Content (High) := Content (High) ** Content (High+1);
        when others  => null;  -- impossible
      end case;
    end loop;
    return Content (1);
  end Process;

  function Evaluate (Expression: String; Coefficients: Values) return Integer is
    Postfix: String renames Infix_to_Postfix (Scan (Expression));
  begin
    Ada.Text_IO.Put_Line ("Ada exression " & Expression & " transformed into " & Postfix);
    return Process (Postfix, Coefficients);
  end Evaluate;

end UPN;
