with Ada.Text_IO, Ada.Integer_Text_IO;
use  Ada.Text_IO, Ada.Integer_Text_IO;

with Ternary_Representations.Classification,
     Ternary_Representations.Put;
use  Ternary_Representations.Classification, Ternary_Representations;

procedure Find_Counterfeit is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      17 August 2022
  --====================================================================
  -- In the range 1 .. (3**n-3)/2, any ternary is either clockwise or
  -- anticlockwise.
  -- Determine for all coins in the range the trit pattern according to
  -- the algorithm when they are heavy or light.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  17.08.2022 Created for Basic Ada Course
  --====================================================================

  Max_Power: constant := 3;
  Max_Coins: constant := (3**(Max_Power+1)-3)/2;

  subtype Coin  is Positive range 1 .. Max_Coins;  -- the coins to check
  subtype Power is Natural  range 0 .. Max_Power;

  Coins: array (Coin) of Standard_Ternary (Power);  -- ternary representation

  Heavy, Light: array (Coin) of Standard_Ternary (Power);  -- the culprit's trit pattern

begin

  To_Ternary:
  for Piece in Coins'Range loop

    Coins (Piece) := to_Standard_Ternary (Piece, Max_Power);

    if Direction (Coins (Piece)) = Counterclockwise then
      Coins (Piece) := Twos_Complement (@);
    end if;

    --Put (Piece);  Put (" => ");  Put (Coins (Piece), Zero_Power_First => False);  New_Line;

  end loop To_Ternary;

  -- Now all kinds of trits (0, 1, 2) are evenly distributed (Max_Coins/3) over all coins in each power.
  -- Why? How to prove?

  Find_Culprit:
  declare

    Left_Scale, Right_Scale: array (1 .. Max_Coins/3) of Coin;  -- the number of coins on the scales
    Left_Level, Right_Level: Natural range 0 .. Max_Coins/3;

    function On_Left_Scale (Piece: Coin) return Boolean is
    begin
      for L of Left_Scale loop
        if L = Piece then
          return True;
        end if;
      end loop;
      return False;
    end On_Left_Scale;

    function On_Right_Scale (Piece: Coin) return Boolean is
    begin
      for R of Right_Scale loop
        if R = Piece then
          return True;
        end if;
      end loop;
      return False;
    end On_Right_Scale;

  begin

    Weighings:
    for P in Power loop

      Left_Level  := 0;
      Right_Level := 0;

      Distribute_Coins:
      for Piece in Coins'Range loop
        case Coins (Piece) (P) is
          when  0 => Left_Level  := @ + 1;  Left_Scale  (Left_Level)  := Piece;
          when  1 => null;
          when  2 => Right_Level := @ + 1;  Right_Scale (Right_Level) := Piece;
          when -1 => null;  -- impossible
        end case;
      end loop Distribute_Coins;

      -- Now the balance scale is filled for the power P.
      -- Assume for each coin that it is either light or heavy and calculate the corresponding trit:
      -- left  scale down => 0,
      -- right scale down => 2,
      -- scales balance   => 1.

      for Piece in Coins'Range loop
        if On_Left_Scale (Piece) then
          Heavy (Piece) (P) := 0;  -- left down
          Light (Piece) (P) := 2;  -- right down
        elsif On_Right_Scale (Piece) then
          Heavy (Piece) (P) := 2;  -- right down
          Light (Piece) (P) := 0;  -- left down
        else
          Heavy (Piece) (P) := 1;
          Light (Piece) (P) := 1;
        end if;
      end loop;

    end loop Weighings;

    -- Now all weighing is done.

  end Find_Culprit;

  -- Why does this algorithm work?

  Output_Culprits_Trit_Pattern:
  declare
    C: Ada.Text_IO.Positive_Count;
  begin
    for Piece in Coins'Range loop
      Put (Piece);  Put (" => ");
      C := Col;     Put (Heavy (Piece), Zero_Power_First => False);
      Set_Col (C);  Put (Light (Piece), Zero_Power_First => False);
      New_Line;
    end loop;
  end Output_Culprits_Trit_Pattern;

end Find_Counterfeit;
