package Ternary_Representations is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      17 August 2022
  --====================================================================
  -- Different number representations in base 3. Ternary digits are
  -- called trits.
  --
  -- Notation: A number is represented as an array of trits, where The
  -- index is the power of 3.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  17.08.2022 Created for Basic Ada Course
  --====================================================================

  pragma Assertion_Policy (Check);

  -- Extended trits

  subtype Trit is Integer range -1 .. 2;

  type Ternary is array (Natural range <>) of Trit
    with Dynamic_Predicate => Ternary'First = 0;

  function Sum (T: Ternary) return Positive;

  -- Standard representation:
  --
  -- (1, 2, 0, 2) = 1*3**0 + 2*3**1 + 2*3**3 = 61 = 3#2021#

  subtype Standard_Trit is Trit range 0 .. 2;

  subtype Standard_Ternary is Ternary
    with Dynamic_Predicate => (for all S of Standard_Ternary => S in Standard_Trit);

  function to_Standard_Ternary (Number: Positive; Max_Power: Positive) return Standard_Ternary
    with Pre => Number < 3**(Max_Power+1);

  function Twos_Complement (S: Standard_Ternary) return Standard_Ternary;

  -- Bilateral representation:
  --
  -- (1, -1, 1, -1, 1) = 1*3**0 - 1*3**1 + 1*3**2 - 1*3**3 + 1*3**4 = 61;

  subtype Bilateral_Trit is Trit range -1 .. 1;

  subtype Bilateral_Ternary is Ternary
    with Dynamic_Predicate => (for all B of Bilateral_Ternary => B in Bilateral_Trit);

  function to_Bilateral_Ternary (Number: Positive; Max_Power: Positive) return Bilateral_Ternary
    with Pre => Number <= (3**(Max_Power+1)-1)/2;

end Ternary_Representations;
