package body DNA_Encoding is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   2.0
  -- Date      19 January 2018
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    2.0  19.01.2018 English version
  --  C.G.    1.0  02.04.2013 Created for Ada course
  --====================================================================

  Virtual_Start_Nucleotide: constant Nucleotide := A;

  subtype Trit    is Character range '0' .. '2';
  subtype Trits   is String (1 .. 6);  -- only '0' .. '2'
  subtype Snippet is DNA (Trits'Range);

  package Map is

    Trit_to_Nucleotide: constant array (Nucleotide, Trit) of Nucleotide :=
      (A => ('0' => C, '1' => G, '2' => T),
       C => ('0' => G, '1' => T, '2' => A),
       G => ('0' => T, '1' => A, '2' => C),
       T => ('0' => A, '1' => C, '2' => G));
    Nucleotide_to_Trit: constant array (Nucleotide, Nucleotide) of Trit :=
      (A => (A => '0', C => '0', G => '1', T => '2'),
       C => (C => '0', G => '0', T => '1', A => '2'),
       G => (G => '0', T => '0', A => '1', C => '2'),
       T => (T => '0', A => '0', C => '1', G => '2'));

    function Image (C: Character) return Trits;  -- to base 3 without base indication
    function Value (T: Trits    ) return Character;

  end Map;

  package body Map is separate;

  function Encode (Text: String) return DNA is
    Predecesssor: Nucleotide := Virtual_Start_Nucleotide;
    Code        : DNA (1 .. 6 * Text'Length);
    function Encode (Char: Trits) return Snippet is
      S: Snippet;
    begin
      for I in Char'Range loop
        S (I) := Map.Trit_to_Nucleotide (Predecesssor, Char (I));
        Predecesssor := S (I);
      end loop;
      return S;
    end Encode;
  begin
    for T in Text'Range loop
      Code (6 * (T - Text'First) + 1 .. 6 * (T - Text'First) + 6) := Encode (Map.Image (Text (T)));
    end loop;
    return Code;
  end Encode;

  function Decode (Code: DNA) return String is
    Predecesssor: Nucleotide := Virtual_Start_Nucleotide;
    Plain       : String (1 .. Code'Length / 6);
    function Decode (S: Snippet) return Character is
      Char: Trits;
    begin
      for I in Char'Range loop
        Char (I) := Map.Nucleotide_to_Trit (Predecesssor, S (I));
        Predecesssor := S (I);
      end loop;
      return Map.Value (Char);
    end Decode;
  begin
    for P in 1 .. Plain'Length loop
      Plain (P) := Decode (Code (6 * (P - 1) + 1 .. 6 * (P - 1) + 6));  -- slice slides
    end loop;
    return Plain;
  end Decode;

end DNA_Encoding;
