package DNA_Encoding is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   3.0
  -- Date      9 March 2023
  --====================================================================
  -- Exercise according to:
  -- Datenspeicher fr die Ewigkeit (Data store for the eternity),
  -- Jan Dnges; Spektrum der Wissenschaft, April 2013, page 16ff
  --
  -- Write a package for encoding and decoding of messages as a
  -- chain of the four DNA nucleotides
  -- Adenie, Cytosine, Guanine und Thymine.
  --
  -- Represent Character'Pos as a ternary number and encode each
  -- trit as follows:
  --
  --               next Trit
  -- Predecessor    0  1  2
  --      A         C  G  T
  --      C         G  T  A
  --      G         T  A  C
  --      T         A  C  G
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    3.0  09.03.2023 Added Dynamic_Predicate and Post
  --  C.G.    2.0  18.01.2018 English version
  --  C.G.    1.0  05.03.2013 Created for Ada course
  --====================================================================

  pragma Assertion_Policy (Check);

  type Nucleotide is (A, C, G, T);
  type DNA is array (Positive range <>) of Nucleotide
    with Dynamic_Predicate => DNA'First = 1;

  function Encode (Text: String) return DNA;
  function Decode (Code: DNA   ) return String
    with Post => Decode'Result'First = 1;

end DNA_Encoding;
